@@ -286,9 +286,18 @@ class HiveQueryTask(luigi.contrib.hadoop.BaseHadoopJobTask):
         """
         return luigi.configuration.get_config().get('hive', 'hiverc-location', default=None)
 
+    def hivevars(self):
+        """
+        Returns a dict of key=value settings to be passed along
+        to the hive command line via --hivevar.
+        This option can be used as a separated namespace for script local variables.
+        See https://cwiki.apache.org/confluence/display/Hive/LanguageManual+VariableSubstitution
+        """
+        return {}
+
     def hiveconfs(self):
         """
-        Returns an dict of key=value settings to be passed along
+        Returns a dict of key=value settings to be passed along
         to the hive command line via --hiveconf. By default, sets
         mapred.job.name to task_id and if not None, sets:
 
@@ -342,6 +351,25 @@ class HiveQueryRunner(luigi.contrib.hadoop.JobRunner):
                     except FileAlreadyExists:
                         pass
 
+    def get_arglist(self, f_name, job):
+        arglist = load_hive_cmd() + ['-f', f_name]
+        hiverc = job.hiverc()
+        if hiverc:
+            if isinstance(hiverc, str):
+                hiverc = [hiverc]
+            for rcfile in hiverc:
+                arglist += ['-i', rcfile]
+        hiveconfs = job.hiveconfs()
+        if hiveconfs:
+            for k, v in six.iteritems(hiveconfs):
+                arglist += ['--hiveconf', '{0}={1}'.format(k, v)]
+        hivevars = job.hivevars()
+        if hivevars:
+            for k, v in six.iteritems(hivevars):
+                arglist += ['--hivevar', '{0}={1}'.format(k, v)]
+        logger.info(arglist)
+        return arglist
+
     def run_job(self, job, tracking_url_callback=None):
         if tracking_url_callback is not None:
             warnings.warn("tracking_url_callback argument is deprecated, task.set_tracking_url is "
@@ -354,18 +382,7 @@ class HiveQueryRunner(luigi.contrib.hadoop.JobRunner):
                 query = query.encode('utf8')
             f.write(query)
             f.flush()
-            arglist = load_hive_cmd() + ['-f', f.name]
-            hiverc = job.hiverc()
-            if hiverc:
-                if isinstance(hiverc, str):
-                    hiverc = [hiverc]
-                for rcfile in hiverc:
-                    arglist += ['-i', rcfile]
-            if job.hiveconfs():
-                for k, v in six.iteritems(job.hiveconfs()):
-                    arglist += ['--hiveconf', '{0}={1}'.format(k, v)]
-
-            logger.info(arglist)
+            arglist = self.get_arglist(f.name, job)
             return luigi.contrib.hadoop.run_and_track_hadoop_job(arglist, job.set_tracking_url)
 
 
