@@ -99,6 +99,12 @@ var LuigiAPI = (function() {
         });
     };
 
+    LuigiAPI.prototype.getTaskProgressPercentage = function(taskId, callback) {
+        return jsonRPC(this.urlRoot + "/get_task_progress_percentage", {task_id: taskId}, function(response) {
+            callback(response.response);
+        });
+    };
+
     LuigiAPI.prototype.getRunningTaskList = function(callback) {
         return jsonRPC(this.urlRoot + "/task_list", {status: "RUNNING", upstream_status: "", search: searchTerm()}, function(response) {
             callback(flatten(response.response));
@@ -152,5 +158,26 @@ var LuigiAPI = (function() {
         });
     }
 
+    LuigiAPI.prototype.pause = function() {
+        jsonRPC(this.urlRoot + '/pause');
+    }
+
+    LuigiAPI.prototype.unpause = function() {
+        jsonRPC(this.urlRoot + '/unpause');
+    }
+
+    LuigiAPI.prototype.isPaused = function(callback) {
+        jsonRPC(this.urlRoot + "/is_paused", {}, function(response) {
+            callback(!response.response.paused);
+        });
+    }
+
+    LuigiAPI.prototype.updateResource = function(resource, n, callback) {
+        var data = {'resource': resource, 'amount': n};
+        jsonRPC(this.urlRoot + "/update_resource", data, function(response) {
+            callback();
+        });
+    }    
+
     return LuigiAPI;
 })();
